@@ -1,18 +1,20 @@
 import requests
 import json
+import pprint
 import os
 import base64
 import xml.etree.ElementTree as ET
 import platform
 import yaml
+from .common import get_env
 from ..common import prettyllog
 
 
 netbox_url = os.environ.get('NETBOX_URL')
 netbox_token = os.environ.get('NETBOX_TOKEN')
 
-NETBOX_URL = os.getenv("NETBOX_API_URL")
-NETBOX_TOKEN = os.getenv("NETBOX_API_TOKEN")
+NETBOX_URL = os.getenv("KALM_NETBOX_API_URL")
+NETBOX_TOKEN = os.getenv("KALM_NETBOX_TOKEN")
 
 ssh_config_template = """
 Host {hostname}
@@ -23,6 +25,359 @@ Host {hostname}
     Port 22
     {proxy_jump}
 """
+
+
+def get_virtual_server_id(servername, env):
+        prettyllog("netbox", "get", "virtual server id", servername, "000" , "getting virtual server id", severity="INFO")
+        url = env['KALM_NETBOX_URL'] + "/api/virtualization/virtual-machines/?name=" + servername
+        headers = {'Authorization': 'Token ' + env['KALM_NETBOX_TOKEN'],
+                   'Accept': 'application/json',
+                   'Content-Type': 'application/json'
+                }
+        r = requests.get(url, headers=headers, verify=env['KALM_NETBOX_SSL'])
+        if r.status_code == 200:
+                data = r.json()
+                if data['count'] == 1:
+                        prettyllog("netbox", "get", "virtual server id", servername, r.status_code , "virtual server id found", severity="INFO")
+                        return data['results'][0]['id']
+                else:
+                        prettyllog("netbox", "get", "virtual server id", servername, r.status_code , "unable to get virtual server id", severity="ERROR")
+                return False
+        else:
+                prettyllog("netbox", "get", "virtual server id", servername, r.status_code , "unable to get virtual server id", severity="ERROR")
+                return False
+  
+
+def get_virtual_server_tags(serverid, env):
+    prettyllog("netbox", "get", "virtual server tags", serverid, "000" , "getting virtual server tags", severity="INFO")
+    url = env['KALM_NETBOX_URL'] + "/api/virtualization/virtual-machines/" + str(serverid)
+    headers = {'Authorization': 'Token ' + env['KALM_NETBOX_TOKEN'],
+               'Accept': 'application/json',
+               'Content-Type': 'application/json'
+            }
+    mytags = []
+    r = requests.get(url, headers=headers, verify=env['KALM_NETBOX_SSL'])
+    if r.status_code == 200:
+        data = r.json()
+        for tag in data['tags']:
+            mytags.append(tag['name'])
+        prettyllog("netbox", "get", "virtual server tags", serverid, r.status_code , "virtual server tags found", severity="INFO")
+        return mytags
+    else:
+        prettyllog("netbox", "get", "virtual server tags", serverid, r.status_code , "unable to get virtual server tags", severity="ERROR")
+        return mytags   
+    
+def get_virtual_server_tagids(serverid, env):
+    prettyllog("netbox", "get", "virtual server tags", serverid, "000" , "getting virtual server tags", severity="INFO")
+    alltags = get_all_tags(env)
+    mytags = get_virtual_server_tags(serverid, env)
+    mytagids = []
+    for tag in mytags:
+        mytagids.append(alltags[tag])
+    return mytagids
+
+    
+
+
+def get_all_tags(env):
+    prettyllog("netbox", "get", "all tags", "000", "000" , "getting all tags", severity="INFO")
+    url = env['KALM_NETBOX_URL'] + "/api/extras/tags/"
+    headers = {'Authorization': 'Token ' + env['KALM_NETBOX_TOKEN'],
+               'Accept': 'application/json',
+               'Content-Type': 'application/json'
+            }
+    # we need to have multipage support
+    multipage = True
+    alltags = {}
+    while multipage:
+        r = requests.get(url, headers=headers, verify=env['KALM_NETBOX_SSL'])
+        if r.status_code == 200:
+            data = r.json()
+            for tag in data['results']:
+                alltags[tag['name']] = tag['id']
+            if data['next'] == None:
+                multipage = False
+            else:
+                url = data['next']
+        else:
+            multipage = False
+    prettyllog("netbox", "get", "all tags", "000", r.status_code , "all tags found", severity="INFO")
+    return alltags
+
+    
+def get_prefixed_tags(serverid, prefix , env):
+    servertags = get_virtual_server_tags(serverid, env)
+    prefixedtags = []
+    for tag in servertags:
+        if tag.startswith(prefix):
+            prefixedtags.append(tag)
+    return prefixedtags
+
+def get_notprefixed_tags(serverid, prefix , env):
+    servertags = get_virtual_server_tags(serverid, env)
+    prefixedtags = []
+    for tag in servertags:
+        if not tag.startswith(prefix):
+            prefixedtags.append(tag)
+    return prefixedtags
+
+
+
+
+def create_random_color():
+    import random
+    r = lambda: random.randint(0,255)
+    return '%02X%02X%02X' % (r(),r(),r())
+
+def create_tag(tag_name):
+    prettyllog("manage", "netbox", "tag", "new", "000", "Creating tag")
+    url = fix_url("/extras/tags/" )
+    headers = {
+        "Authorization": f"Token {NETBOX_TOKEN}",
+        "Accept": "application/json"
+    }
+    data = {
+        "name": tag_name,
+        "slug": tag_name.lower(),
+        "color": "896818"
+    }
+    response = requests.post(url, headers=headers, json=data, verify=False)
+    if response.status_code == 201:
+        return True
+    else:
+        return False
+    
+def removetagsfromvm(vmname, prefix, env):
+    prettyllog("manage", "netbox", "tag", "new", "000", "Removing tags from vm %s" % vmname)
+    url = fix_url("/virtualization/virtual-machines/?name=%s" % vmname )
+    headers = {
+        "Authorization": f"Token {NETBOX_TOKEN}",
+        "Accept": "application/json"
+    }
+    response = requests.get(url, headers=headers, verify=False)
+    if response.status_code == 200:
+        data = response.json()
+        if len(data['results']) == 1:
+            vmid = data['results'][0]['id']
+            keeptags = get_notprefixed_tags(vmid, prefix, env)
+            url = fix_url("/virtualization/virtual-machines/%s/" % vmid )
+            data = {
+                "tags":  keeptags
+            }
+            headers = {
+                "Authorization": f"Token {NETBOX_TOKEN}",
+                 "Accept": "application/json"
+                }
+            response = requests.patch(url, headers=headers,  verify=False)
+    
+def addtagtovm(vmid, tagids, env):
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tags to vm %s" % (vmid))
+    mytags = get_virtual_server_tagids(vmid, env)
+    prettyllog("manage", "netbox", "tag", "new", "000", "Current tags %s" % (mytags))
+    url = fix_url("/virtualization/virtual-machines/%s/" % vmid )
+    data = {
+        "tags":  mytags + tagids
+    }
+    headers = {'Authorization': 'Token ' + env['KALM_NETBOX_TOKEN']}
+    response = requests.patch(url, headers=headers, json=data, verify=False)
+    if response.status_code == 200:
+        return True
+    else:
+        return False
+    
+
+
+
+
+
+
+            
+
+
+
+                
+def addvmwaretags(serverid, details, env):
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tags to vm %s" % serverid)
+    print("----------------------------------------------------------")
+    pprint.pprint(details)
+    print("----------------------------------------------------------")
+    # guestId': 'debian10_64Guest',
+# 'hostName': 'acs-sndb2.greennet.gl',
+# 'hwVersion': 'vmx-19',
+# 'ipAddress': '194.177.224.46',
+# 'memory': 16384,
+# 'numCpu': 8,
+# 'numEthernetCards': 1,
+# 'numVirtualDisks': 2,
+# 'overallStatus': 'green',
+# 'toolsRunningStatus': 'guestToolsRunning',
+# 'toolsStatus': 'toolsOk',
+# 'toolsVersionStatus': 'guestToolsUnmanaged',
+# 'toolsVersionStatus2': 'guestToolsUnmanaged',
+# 'uuid': '4212ce39-15d2-ca2a-8c72-c0ee0676a6ee',
+# 'vmPathName': '[vsan_S3_Linux] '
+#               '7352c963-5404-41b8-66f6-0025b531a107/acs-db2.greennet.gl.vmx'}
+
+
+    prefix = "vmware_guestId_"
+    # guestId
+    guestId = "%s%s" % (prefix, details['guestId'])
+    
+    create_tag(guestId)
+
+    prefix = "vmware_hwVersion_"
+    hwVersion = "%s%s" % (prefix, details['hwVersion'])
+    create_tag(hwVersion)
+
+    prefix = "vmware_overallStatus_"
+    overallStatus = "%s%s" % (prefix, details['overallStatus'])
+    create_tag(overallStatus)
+
+    prefix = "vmware_toolsRunningStatus_"
+    toolsRunningStatus = "%s%s" % (prefix, details['toolsRunningStatus'])
+    create_tag(toolsRunningStatus)
+
+    prefix = "vmware_toolsVersionStatus_"
+    toolsVersionStatus = "%s%s" % (prefix, details['toolsVersionStatus'])
+    create_tag(toolsVersionStatus)
+
+    prefix = "vmware_toolsVersionStatus2_"
+    toolsVersionStatus2 = "%s%s" % (prefix, details['toolsVersionStatus2'])
+    create_tag(toolsVersionStatus2)
+
+    prefix = "vmware_vmPathName_"
+    vmPathNamefull = "%s%s" % (prefix, details['vmPathName'])
+    vmPathName = vmPathNamefull.split("]")[1].split("/")[1].replace(".vmx", "").replace(".", "_")
+    create_tag(vmPathName)
+
+    prefix = "vmware_toolsStatus_"
+    toolStatus ="%s%s" % (prefix, details['toolsStatus'])
+    create_tag(toolStatus)
+
+
+    alltags = get_all_tags(env)
+
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tags to vm %s" % (serverid))
+    mynewids = []
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolStatus, serverid))
+    try:
+        mynewids.append(alltags[toolStatus])
+    except:
+        pass
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolStatus, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (vmPathName, serverid))
+    try:
+        mynewids.append(alltags[vmPathName])
+    except:
+        pass
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (vmPathName, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolsVersionStatus2, serverid))
+    try:
+        mynewids.append(alltags[toolsVersionStatus2])
+    except:
+        pass
+
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolsVersionStatus2, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolsVersionStatus, serverid))
+    try:
+        mynewids.append(alltags[toolsVersionStatus])
+    except:
+        pass
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolsVersionStatus, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolsRunningStatus, serverid))
+    try:
+        mynewids.append(alltags[toolsRunningStatus])
+    except:
+        pass
+
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolsRunningStatus, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (overallStatus, serverid))
+    try:
+        mynewids.append(alltags[overallStatus])
+    except:
+        pass
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (overallStatus, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (hwVersion, serverid))
+    try:
+        mynewids.append(alltags[hwVersion])
+    except:
+        pass
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (hwVersion, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (guestId, serverid))
+    try:
+        mynewids.append(alltags[guestId])
+    except:
+        pass
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (guestId, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (vmPathName, serverid))
+    try:
+        mynewids.append(alltags[vmPathName])
+    except:
+        pass
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (vmPathName, serverid))
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolStatus, serverid))
+    try:
+        addtagtovm(serverid, mynewids, env)
+    except:
+        pass
+
+    prettyllog("manage", "netbox", "tag", "new", "000", "Adding tag %s to vm %s" % (toolStatus, serverid))
+
+
+    
+
+
+
+
+
+
+    
+
+
+
+           
+
+
+def get_netbox_inventory_from_tag(tag):
+    prettyllog("manage", "netbox", "inventory", "new", "000", "Getting inventory from tag %s" % tag)
+    # This is the format of the inventory string :           'inventory': '[website]\n172.18.8.40\n172.18.8.41',
+
+    invstring = ""
+
+    invstring = '[%s]\n' % tag
+
+    url = fix_url("/virtualization/virtual-machines/?tag=%s" % tag)
+    headers = {
+        "Authorization": f"Token {NETBOX_TOKEN}",
+        "Accept": "application/json"
+    }
+    response = requests.get(url, headers=headers)
+    if response.status_code == 200:
+        data = response.json()
+        for server in data['results']:
+            prettyllog("manage", "netbox", "inventory", "new", "000", "Found server %s" % server['name'], severity="DEBUG")
+            invstring += "%s\n" % server['name']
+        while data['next'] != None:
+            response = requests.get(data['next'], headers=headers)
+            if response.status_code == 200:
+                data = response.json()
+                for server in data['results']:
+                    prettyllog("manage", "netbox", "inventory", "new", "000", "Found server %s" % server['name'], severity="DEBUG")
+                    invstring += "%s\n" % server['name']
+            else:
+                return None
+        return str(invstring)
+    else:
+        return None
+    
+
+
+
+
+
+
+
+
 def vizulize(args):
     cluseters = get_clusters()
     vms = get_virtual_machines()
@@ -110,8 +465,8 @@ def get_manufacturer_id(manufacturer_name):
     except:
         return None
         
-def get_virtual_machine_id(vm_name):
-    vms = get_virtual_machines()
+def get_virtual_machine_id(vm_name, env):
+    vms = get_virtual_machines(env)
     try:
         return vms[vm_name]
     except:
@@ -161,10 +516,6 @@ def create_manufacturer(manufacturer_name):
 
     url = fix_url("/dcim/manufacturers/")
     response = requests.post(url, headers=headers, json=data)
-    print(response)
-    print(response.json())
-    print(response.status_code)
-    print(response.content)
     if response.status_code == 200:
         return True
     else:
@@ -257,10 +608,6 @@ def create_device_type(device_type_name):
         return True
     url = fix_url("/dcim/device-types/")
     response = requests.post(url, headers=headers, json=data)
-    print(response)
-    print(response.json())
-    print(response.status_code)
-    print(response.content)
     if response.status_code == 200:
         return True
     else:
@@ -356,10 +703,6 @@ def create_role():
         return True
     url = fix_url("/dcim/device-roles/")
     response = requests.post(url, headers=headers, json=data)
-    print(response)
-    print(response.json())
-    print(response.status_code)
-    print(response.content)
     if response.status_code == 200:
         return True
     else:
@@ -420,10 +763,6 @@ def create_site(site_name):
         return True
     url = fix_url("/dcim/sites/")
     response = requests.post(url, headers=headers, json=data)
-    print(response)
-    print(response.json())
-    print(response.status_code)
-    print(response.content)
     if response.status_code == 200:
         return True
     else:
@@ -467,10 +806,6 @@ def create_manufacturer(manufacturer_name):
     
     url = fix_url("/dcim/manufacturers/")
     response = requests.post(url, headers=headers, json=data)
-    print(response)
-    print(response.json())
-    print(response.status_code)
-    print(response.content)
     if response.status_code == 200:
         return True
     else:
@@ -552,10 +887,6 @@ def create_platform(platform_name):
         return True
     url = fix_url("/dcim/platforms/")
     response = requests.post(url, headers=headers, json=data)
-    print(response)
-    print(response.json())
-    print(response.status_code)
-    print(response.content)
     if response.status_code == 200:
         return True
     else:
@@ -713,7 +1044,6 @@ def create_iprange(iprange_description = "default"):
 
     url = fix_url("/ipam/ip-ranges/")
     response = requests.post(url, headers=headers, json=data)
-    print(response.content)
     if response.status_code == 200:
         return True
     else:
@@ -961,7 +1291,6 @@ def add_device():
         url = fix_url("/dcim/devices/")
         response = requests.post(url, headers=headers, json=data)
         prettyllog("manage", "netbox", "device", "new", "000",  "Api call status is %s" % response.status_code)
-        print(response.content)
         if response.status_code == 200:
             return True
         else:
@@ -1122,9 +1451,9 @@ def add_vm():
 
 
 def fix_url(apiurl):
-    nburl=os.environ.get('NETBOX_API_URL')
+    nburl=os.environ.get('KALM_NETBOX_URL')
     if nburl == None:
-        print("No NETBOX_API_URL provided")
+        print("No KALM_NETBOX_URL provided")
         return False
 
     if nburl.endswith("/"):
@@ -1191,7 +1520,6 @@ def get_roles():
     response = requests.get(url, headers=headers)
     roles = response.json()
     for role in roles["results"]:
-        print(role)
         try:
             if returnroles[role["name"]]:
                 print("Duplicate role name")
@@ -1211,7 +1539,6 @@ def get_device_types():
     response = requests.get(url, headers=headers)
     types = response.json()
     for type in types["results"]:
-        print(type)
         try:
             if returntypes[type["display"]]:
                 print("Duplicate type name")
@@ -1404,28 +1731,39 @@ def get_virtual_machine(id):
     vm = response.json()
     return vm
 
-def get_virtual_machines():
+def get_virtual_machines(env = get_env()):
     returnvms = {}
-    headers = {
-        "Authorization": f"Token {NETBOX_TOKEN}",
-        "Accept": "application/json"
-    }
+    headers = {'Authorization': 'Token ' + env['KALM_NETBOX_TOKEN']}
     url = fix_url("/virtualization/virtual-machines/")
-    response = requests.get(url, headers=headers)
-    vms = response.json()
-    for vm in vms["results"]:
-        try:
-            if returnvms[vm["name"]]:
-                print("Duplicate vm name")
-        except:
-           returnvms[vm["name"]] = vm["id"]
+    morepages = True
+    while morepages: 
+        response = requests.get(url, headers=headers, verify=env['KALM_NETBOX_SSL'])
+        vms = response.json()
+        for vm in vms["results"]:
+            try:
+                if returnvms[vm["name"]]:
+                    dupvm=True
+            except:
+                returnvms[vm["name"]] = vm["id"]
+        # we need to check if there are more pages 
+        if vms['next'] is not None:
+            url = vms['next']
+            pprint.pprint(vms['next'])
+        else:
+            morepages = False 
     return returnvms
 
 
     
 def netboxdata(args):
+    prettyllog("neboxdata", "collect", "data", "kalm", "000", "Collecting data", "info")
+    prettyllog("neboxdata", "collect", "data", "kalm", "000", "Collecting cluster data", "info")
     clusters = get_clusters()
-    vms = get_virtual_machines()
+    prettyllog("neboxdata", "collect", "data", "kalm", "000", "Collecting my environment", "info")
+    myenv = get_env()
+    prettyllog("neboxdata", "collect", "data", "kalm", "000", "Collecting my virtualmachines", "info")
+    vms = get_virtual_machines(myenv)
+
     vm_data = []
     for vm in vms:
         vmdata = get_virtual_machine(vms[vm])
@@ -1446,7 +1784,6 @@ def netboxdata(args):
     data = {
         "virtual_machines": vm_data
     }
-    print(json.dumps(data, indent=2))
     return data
 
 def ansible_inventory(args):
@@ -1489,12 +1826,13 @@ def ansible_inventory(args):
                 "memory_mb": vm["memory"]
             }
 
-    print(yaml.dump(data))
-
 
 def sshconfig(args):    
+    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "Starting ssh config update", "INFO")
     data = netboxdata(args)
+    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "Got netbox data", "INFO")
     virtual_machines = data["virtual_machines"]
+    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "Got virtual machines", "INFO")
     ssh_config_entries = [generate_ssh_config_entry(vm) for vm in virtual_machines]
     ssh_config_content = "\n".join(ssh_config_entries)
     MYHOME = os.getenv("HOME")
@@ -1516,14 +1854,30 @@ def sshconfig(args):
         open(configfilemaster, "w").write("Include ~/.ssh/conf.d/*\n")
     MYHOME = os.getenv("HOME")
     configfile = os.path.expanduser(MYHOME + "/.ssh/conf.d/kalm.conf")
+    #create an empty file if its missing
+    touch(configfile)
+    touch(configfilemaster) 
+    
+
+
+
     os.chmod(configfilemaster, 0o0600)
     os.chmod(configfile, 0o0600)
+    # open config file and create it if its missing
     open(configfile, "w").write(ssh_config_content)
-    print(ssh_config_content)
+    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "new ssh config written", "INFO")
 
 
 
 
+def touch(file_path):
+    try:
+        # Open the file in append mode, which creates the file if it doesn't exist
+        with open(file_path, "a"):
+            os.utime(file_path, None)  # Update the file's access and modification timestamps
+        print(f"File '{file_path}' created or updated successfully.")
+    except Exception as e:
+        print(f"Error: {e}")
 
 
 def generate_ssh_config_entry(vm):
